import datetime
import json
import os
import random
import typing

import dateutil.parser as date_parser
import requests
from bs4 import BeautifulSoup

# Location of package-included user agents
DATA_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), '..', 'data')
USER_AGENT_FILE = os.path.join(DATA_DIR, 'user_agents.json')


def load_user_agents(filepath: str) -> typing.List[str]:
    """
    Loads list of user agents
    Args:
        filepath: path to user agent file

    Returns:
        List of user agents

    """
    user_agents = []

    with open(filepath) as f:
        user_agents = json.load(f)

    return user_agents


def scrape_profile(username: str, user_agents: typing.List[str]) -> typing.List[typing.Dict]:
    """
    Scrapes public Venmo profile for specified username

    Args:
        username:
        user_agents:

    Returns:

    """
    try:
        r = requests.get(f"https://venmo.com/{username}",
                         headers={"User-Agent": random.choice(user_agents)})
    except requests.exceptions.ConnectionError:
        print("Error, unable to connect to host... check your network connection")
        return 1

    soup = BeautifulSoup(r.text, "lxml")
    profile_data = []

    transactions = soup.find_all("div", attrs={"class": "single-payment content-wrap"})

    for transaction in transactions:
        send, recv = transaction.find_all("a")
        send, recv = send.text, recv.text
        message = transaction.find_all("div", attrs={"class": "paymentpage-text m_five_t"})[0].text
        date = date_parser.parse(transaction.find_all("div", attrs={"class": "date"})[0].text.split('-')[0], fuzzy=True)
        export_message = f"{send} paid {recv} on {date.replace(microsecond=0).isoformat()} for {message}"
        profile_data.append({"sender": send,
                             "recipient": recv,
                             "date": date.replace(microsecond=0).isoformat(),
                             "export_message": export_message})

    return profile_data


def save_data(data: typing.List[typing.Dict], filename: str):
    """
    Saves scraped profile data to file
    Args:
        data:
        filename:

    Returns:

    """
    now = datetime.datetime.now().replace(microsecond=0).isoformat()
    with open(f"{filename}-{now}", "w") as f:
        f.write(json.dumps(data))



import click

from .main import USER_AGENT_FILE, load_user_agents, save_data, scrape_profile


@click.command()
@click.argument('username')
@click.option('--filename', default='venmo-txn', help='Filename of which to save scraped data')
def cli(username: str, filename: str):
    """
    Extract Venmo transactions from a profile with one command


    """
    user_agents = load_user_agents(USER_AGENT_FILE)
    profile_data = scrape_profile(username, user_agents)
    click.echo(f"Extracted {len(profile_data)} transactions for username '{username}'")
    save_data(profile_data, filename)

if __name__ == '__main__':
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['venpo']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.1,<5.0.0',
 'click>=7.1.2,<8.0.0',
 'lxml>=4.5.2,<5.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'requests>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['venpo = venpo.cli:cli']}

setup_kwargs = {
    'name': 'venpo',
    'version': '0.2.1',
    'description': 'Extract Venmo transactions from a profile with one command',
    'long_description': "# venpo\n\nExtract Venmo transactions from a profile with one command\n\n## Demo\n\nMake demo here\n\n## About\n\nA simple command line tool to extract public Venmo transactions from a specified user profile.\n\nvenpo will access a Venmo user's profile page, extract the transactions, and save to a JSON file.\n\n## Use Cases\n\n1. Save a history of your own or a friend's Venmo transactions\n2. Transaction monitoring for OSINT (open source intelligence) purposes\n3. Collecting data to train your own ML model\n\n\n## Installation\n\n```shell script\n$ pip install venpo --upgrade\n```\n\n## Usage\n\n```shell script\n$ venpo $username\n```",
    'author': 'Marc Ford',
    'author_email': 'mrfxyz567@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mfdeux/venpo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
